 /*
 * MQTT Dashboard for doing $stuff in CCCFr hackspace (cccfr.de)
 *
 * https://wiki.cccfr.de/infrastruktur:mqtt
 *
 * License: do the fuck you want with this.
 *
 */

let locStore        = false; // window.localStorage available 0|1
let showInfoboard   = false;
let showRooms       = false;
let subscibedTopics = false;

// document fully loaded trigger
var domReady = (callback) => {
    if (document.readyState != "loading") callback();
    else document.addEventListener("DOMContentLoaded", callback);
};


function localeSort(items, it) {
    return items.slice().sort((a, b) => {
        const valueA = String(it(a))
        const valueB = String(it(b))
        return valueA.localeCompare(valueB, undefined, {
            ignorePunctuation: true,
            numeric: true,
            sensitivity: 'base',
        })
    })
}

// document fully loaded funciton
domReady(() => {
    locStore = storageAvailable();

    let params = (new URL(document.location)).searchParams;
    showInfoboard = params.has('infoboard');

    if (showInfoboard === true) {
        document.getElementById('panel-infoscreen').classList.remove('hard-hidden');
    }

    if (locStore === true) {
        if (!localStorage.getItem('foldout')) {
            localStorage.setItem('foldout', JSON.stringify(initialFold));
        }
        if (!localStorage.getItem('showRooms')) {
            localStorage.setItem('showRooms', JSON.stringify(initialRooms));
        }
        if (!localStorage.getItem('subscibedTopics')) {
            localStorage.setItem('subscibedTopics', JSON.stringify(monTopicsInit));
        }
    }

    showRooms = (locStore === true) ? JSON.parse(localStorage.getItem('showRooms')):initialRooms;
    subscibedTopics = (locStore === true) ? JSON.parse(localStorage.getItem('subscibedTopics')):monTopicsInit;

    mqttDashboard();
});

function getRandomIdentifier(length = 16) {
    let r = ''
    for (let i = 0; i < length; i++) {
        r += Math.floor(Math.random() * 16).toString(16)
    }
    return r
}

const randomIdentifier = getRandomIdentifier()
const mqttClient = MQTT_CLIENT.replace('%s', randomIdentifier)

const mqttUri = typeof MQTT_URI === 'undefined'
    ?  `ws://${MQTT_HOST}:${MQTT_PORT}/mqtt`
    : MQTT_URI

function mqttDashboard() {
    let client          = null;

    /*
    * MQTT Client initilisation & foo
    */
    function connect(){
        // see https://www.eclipse.org/paho/files/jsdoc/Paho.MQTT.Client.html for details
        addRoomSettings();
        addTopicSettings();

        addConnectionInfo();
        client = new Paho.MQTT.Client(mqttUri, mqttClient);
        console.info(`Connecting to MQTT Broker at ${mqttUri} as client ${mqttClient}`)

        document.getElementById('network-info').textContent = `connecting to ${mqttUri}`;

        // callback handlers
        client.onConnectionLost = onConnectionLost;
        client.onMessageArrived = message => {
            try {
                onMessageArrived(message);
            } catch (error) {
                console.error(error, message);
            }
        }

        var options = {
            onSuccess: onConnect,
            onFailure: onFail,
            ...(typeof MQTT_OPTIONS !== 'undefined' ? MQTT_OPTIONS || {} : {}),
        };

        client.connect(options);
    }

    function onConnect(context) {
        console.info("Client Connected", context);
        addInfoPanelGroup();
        Object.entries(subscibedTopics).forEach(([key, value]) => {
            topicSubscribe(value);
        });

        document.getElementById('network-info').textContent = `connected to ${mqttUri}`;
        document.getElementById('client-conn-status').remove();
        document.body.classList.replace('offline', 'online');
    }

    function topicSubscribe(topic) {
        // And subscribe to our topics  -- both with the same callback function
        let options = {qos:0};
        client.subscribe(topic, options);
        console.log('subscibed: ', topic);
    }

    function topicUnsubscribe(topic) {
        let options = {};
        client.unsubscribe(topic, options);
        console.log('unsubscibed: ', topic);

        if (/^pwr\//.test(topic)) {
            removeElemtBySelector('.device-obi-socket');
        }

        if (/^lwnd\//.test(topic)) {
            removeElemtBySelector('.device-lwnd');
        }

        if (/^rgb\//.test(topic)) {
            removeElemtBySelector('.device-rgb-led');
        }

        if (/^mpd\//.test(topic)) {
            removeElemtBySelector('.device-mpd');
        }

        if (/^door\/state\//.test(topic)) {
            removeElemtBySelector('#doorstate');
        }

        if (/^info\/laser/.test(topic)) {
            removeElemtBySelector('.lasercutter');
        }

        if (/^info\/members/.test(topic)) {
            removeElemtBySelector('#info-members');
        }

        if (/^info\/bigblue/.test(topic)) {
            removeElemtBySelector('#info-bigblue');
        }

        if (/^info\/hc-temp/.test(topic)) {
            removeElemtBySelector('#hc-temp');
        }

        if (/^info\/sandserver-temp/.test(topic)) {
            removeElemtBySelector('#sand-temp');
        }

        if (/^info\/mate/.test(topic)) {
            removeElemtBySelector('#info-mate');
        }

        if (/^info\/gw/.test(topic)) {
            removeElemtBySelector('.device-gw-network');
        }

        if (/^info\/net\/uplink\//.test(topic)) {
            removeElemtBySelector('.device-info-uplink');
        }

        if (/^info\/co2\//.test(topic)) {
            removeElemtBySelector('.device-info-co2');
        }

        if (/^info\/net\/ip-leases/.test(topic)) {
            document.getElementById('infopanels/ip-leases-group').remove();
        }

        if (/^info\/net\/status/.test(topic)) {
            document.getElementById('infopanels/net-status-group').remove();
        }

        document.querySelectorAll('.device-group').forEach(function(value){
            if (value.querySelectorAll(':scope ul li').length == 0) {
                value.remove();
            };
        });
    }

    function removeElemtBySelector(removeSelector) {
        document.querySelectorAll(removeSelector).forEach(function(value){
            value.remove();
        });
    }

    function onFail(context) {
        console.log("Failed to connect", context);
        document.getElementById('network-info').textContent = `¯\\_(ツ)_/¯ failed connecting to ${mqttUri}`;
        addConnectionInfo();
        document.getElementById('client-conn-status-info').textContent = `failed connecting to ${mqttUri}`;
        document.body.classList.replace('online', 'offline');
    }

    function onConnectionLost(responseObject) {
        if (responseObject.errorCode !== 0) {
            console.log("Connection Lost: " + responseObject.errorMessage);
            console.log(responseObject);
        }
        document.getElementById('network-info').textContent = `¯\\_(ツ)_/¯ connection lost: ${mqttUri}`;

        addConnectionInfo();
        document.getElementById('client-conn-status-info').textContent = `connection lost: ${mqttUri}`;
        document.body.classList.replace('online', 'offline');
    }

    function addConnectionInfo() {
        if (document.getElementById('client-conn-status') == null) {
            document.body.insertAdjacentHTML('afterbegin', `
                <div id="client-conn-status">
                    <button type="button" id="client-conn-status-button">
                            <span>¯\\_(ツ)_/¯</span>
                            <span class="fa  fa-refresh"></span>
                            <span id="client-conn-status-info">connecting to ${mqttUri}</span>
                    </button>
                </div>`);
        }
    }

    function addHackspaceDoor() {
        if (document.getElementById('doorstate') == null) {
            document.getElementById('header').insertAdjacentHTML('beforeend', `
                <div id="doorstate" data-state="open" data-networkstate="online">
                    <button type="button" data-buttontype="door-button" >
                        <span class="fa fa-home fa-fw"></span>
                        <span class="device-state-text"></span>
                        <span class="fa stateicon"></span>
                    </button>
                </div>
            `);
        }
    }

    //  handling incoming messages
    function onMessageArrived(message) {
        // console.log('rx', message);

        // Door State
        if (/^door\/state/.test(message.destinationName)) {
            addHackspaceDoor();
            if (message.payloadString == "open") {
                document.getElementById('doorstate').setAttribute('data-state', 'open');
            }

            if (message.payloadString == "close") {
                document.getElementById('doorstate').setAttribute('data-state', 'close');
            }
        }

        // OBI Sockets
        if (/^pwr\//.test(message.destinationName)) {

            if(/\/tele\/LWT$/.test(message.destinationName)) {
                obiSocketTele(message);
            }

            if(/\/tele\/SENSOR$/.test(message.destinationName)) {
                pwrSensor(message);
            }

            if(/\/stat\/[^/]+$/.test(message.destinationName)) {
                obiSocketStat(message);
            }
        }

        // MPD
        if (/^mpd\//.test(message.destinationName)) {
            if(/\/info$/.test(message.destinationName)) {
                addMpdControl(message);
            }

            if (/\/jinglelist$/.test(message.destinationName)) {
                addJingleBoard(message)
            }
        }

        // screen control
        if (/^lwnd\//.test(message.destinationName)) {
            lwndScreenCtrl(message);
        }

        // RGB control
        if (/^rgb\//.test(message.destinationName)) {
            if(/\/stat\/[^/]+$/.test(message.destinationName)) {
                rgbCtrl(message);
            }
        }

        // Screen control
        if (/^experimental\/thed\/grossundunpraktisch\//.test(message.destinationName)) {
            thedGame(message);
        }

        // Infoscreen Members
        if (/^info\/members/.test(message.destinationName)) {
            addMemberPanel(JSON.parse(message.payloadString));
        }

        // Infoscreen bigblue
        if (/^info\/bigblue/.test(message.destinationName)) {
            addbbbPanel(message);
        }

        // Infoscreen HC-test
        if (/^info\/hc-temp/.test(message.destinationName)) {
            addTempPanel(JSON.parse(message.payloadString));
        }

        // Infoscreen HC-test
        if (/^info\/mate/.test(message.destinationName)) {
            addMatePanel(JSON.parse(message.payloadString));
        }

        // we got LASERS!
        if (/^info\/laser/.test(message.destinationName)) {
            lasershow(JSON.parse(message.payloadString));
        }

        // Network Interfaces
        if (/^info\/gw\//.test(message.destinationName)) {
            addNetworkPanel(message);
        }

        // Network IP-Leases
        if (/^info\/net\/ip-leases/.test(message.destinationName)) {
            addIpPanel(message);
        }

        // Network IP-Leases
        if (/^info\/net\/status/.test(message.destinationName)) {
            addNetStaus(message);
        }

        // Network Uplinks
        if (/^info\/net\/uplink/.test(message.destinationName)) {
            // addIpPanel(message);
            addUplinkPanels(message);
        }

        // CO2 Sensor
        if (/^info\/co2/.test(message.destinationName)) {
            addCo2Sensor(message);
        }
    }


    function sendMqttMessage(topic, payload, retained = false) {
        // console.log(topic, payload);
        let message = new Paho.MQTT.Message(payload);
        message.destinationName = topic;
        message.retained = retained;
        client.send(message);
    }

    function addCo2Sensor(message) {
        addInfoPanelGroup();
        let msg = JSON.parse(message.payloadString);
        let sensor_id = message.destinationName.replace('/','-');
        let result = message.destinationName.match(/[^/]+$/);
        if (document.getElementById(sensor_id) == null) {
            document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `<li id="${sensor_id}" class="device-info-co2"></li>`);
        }
        document.getElementById(sensor_id).innerHTML = `<h3><span class="fa fw fa-cloud device-icon"></span> ${result[0]}:  ${msg.co2}<span class="small">ppm</span>  ${msg.tmp}<span class="small">°C</span>  ${msg.hum}<span class="small">%</span></h3>`;
    }

    function addIpPanel(message) {
        let msg = JSON.parse(message.payloadString);
        if (msg !== 'undefined') {
            let cont = '';
            addInfoPanelGroup();
            domGroupCreation('infopanels/ip-leases', false);
            Object.entries(msg).forEach(([key, value]) => {
               cont += '<li class="ip-lease"><a href="http://' + key + '" title="' + key + ': ' + value + '" target="_blank"><h3>' + key + ': ' + value + '</h3></a></li>';
            })

            document.getElementById('infopanels/ip-leases-group-ul').innerHTML = cont;
        }
    }

    function addNetStaus(message) {
        let msg = JSON.parse(message.payloadString);
        if (msg == 'offline') {
            console.log(msg);
            return;
        }
        if (msg !== 'undefined') {
            let cont = '';
            addInfoPanelGroup();
            domGroupCreation('infopanels/net-status', false);
            if (document.getElementById('net-status-' +msg.ip) == null) {
                document.getElementById('infopanels/net-status-group-ul').insertAdjacentHTML('beforeend', `
                <li id="net-status-${msg.ip}" class="device-net-status">
                </li>
                `);
            }
            document.getElementById('net-status-' +msg.ip).innerHTML = `
            <h3 data-state="${msg.state}"><span class="fa fw fa-laptop device-icon"></span> ${msg.ip}: ${msg.name}</h3>`;
        }
    }

    function addNetworkPanel(message) {
        addInfoPanelGroup();
        let id = message.destinationName.replace('/', '-');
        let msg = JSON.parse(message.payloadString);
        if (document.getElementById(id) == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="${id}" class="device-gw-network">
            </li>
            `);
        }
        document.getElementById(id).innerHTML = `
            <h3><span class="fa fw fa-cloud-upload device-icon"></span> ${msg.interface}: rx: ${msg.rx.ratestring} | tx: ${msg.tx.ratestring}</h3>`;
    }

    function addbbbPanel(message) {
        addInfoPanelGroup();
        if (document.getElementById('info-bigblue') == null) {
            document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="info-bigblue" class="device-info-bigblue">
                <h3>BBB: Keine Daten derzeit</h3>
            </li>
            `);
        }
        try {
            mpdInfo = JSON.parse(message.payloadString);
        } catch(e) {
            console.log('JSON Defekt für: info-bigblue: ' + message.payloadString);
            return;
        }
        document.getElementById('info-bigblue').innerHTML = `
            <h3><span class="fa fw fa-users device-icon"></span> BBB Rooms: ${mpdInfo.rooms}<br>Users: ${mpdInfo.users} (${mpdInfo.mod} Mods), ${mpdInfo.voice} voice and ${mpdInfo.video} video</h3>`;
    }

    function addMemberPanel(message) {
        addInfoPanelGroup();
        if (document.getElementById('info-members') == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="info-members" class="device-info-members">
            </li>
            `);
        }
        document.getElementById('info-members').innerHTML = `
            <h3><span class="fa fw fa-users device-icon"></span> Members: ${message.members}</h3>`;
    }

    function addMatePanel(message) {
        addInfoPanelGroup();
        // message = JSON.parse(message.payloadString);
        if (document.getElementById('info-mate') == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="info-mate" class="device-info-mate">
            </li>
            `);
        }
        document.getElementById('info-mate').innerHTML = `
            <h3><span class="fa fw fa-glass device-icon"></span> ${message.Name}: ${message.Balance} €</h3>`;
    }

    function addUplinkPanels(message) {
        addInfoPanelGroup();
        let msg = JSON.parse(message.payloadString);
        if (document.getElementById('info-uplink-' + msg.interface) == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="info-uplink-${msg.interface}" class="device-info-uplink">
            </li>
            `);
        }
        let state = (msg.ip == 'down') ? 0:1;
        document.getElementById('info-uplink-' + msg.interface).innerHTML = `
            <h3 data-state="${state}"><span class="fa fw fa-cloud-upload device-icon"></span> ${msg.name} - ${msg.ip} - ${msg.interface}</h3>`;
    }

    function addTempPanel(message) {
        addInfoPanelGroup();
        if (document.getElementById('hc-temp') == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="hc-temp" class="device-hc-temp">
            </li>
            `);
        }
        document.getElementById('hc-temp').innerHTML = `
            <h3><span class="fa fw fa-thermometer-full device-icon"></span> ${message.Name}: ${message.Temp}${message.Unit} | Humidity: ${message.Humidity}</h3>`;
    }

    function addSandTempPanel(message) {
        addInfoPanelGroup();
        if (document.getElementById('hc-temp') == null) {
        document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="sand-temp" class="device-sand-temp">
            </li>
            `);
        }
        document.getElementById('sand-temp').innerHTML = `
            <h3><span class="fa fw fa-thermometer-full"></span> ${message.Name}: ${message.Temp}${message.Unit}</h3>`;
    }

    function addInfoPanelGroup() {
        if (document.getElementById('infopanels-group') == null) {
            // let showtarget = (showRooms.includes('infopanels')) ? 'panel-autobtns':'archive';
            let togClass = 'toggle-content toggle-content-hidden';
            let togIconClass = '';
            if (locStore === true) {
                if (JSON.parse(localStorage.getItem('foldout')).includes('infopanels-group')) {
                    togClass = 'toggle-content';
                    togIconClass = 'fa-rotate-90';
                }
            } else {
                if (initialFold.includes('infopanels-group')) {
                    togClass = 'toggle-content';
                    togIconClass = 'fa-rotate-90';
                }
            }
            document.getElementById('panel-autobtns').insertAdjacentHTML('beforeend', `
            <li id="infopanels-group" name="infopanels-group" class="device-group">
                <div class="group-head-single">
                    <h2>
                        <a href="#" title="toggle" class="group-toggle}" data-toggroupid="infopanels-group-ul"><span class="fa fa-fw fa-cog ${togIconClass}"></span> Infos & Settings</a>
                    </h2>
                </div>
                <ul id="infopanels-group-ul" class="group-body ${togClass}">
                </ul>
            </li>`);
        }
    }

    // creating the standard DOM Groups for buttons
    function domGroupCreation(groupPath, buttons = true, buttonType = 'obi_socket_group') {
        let domAr = groupPath.split('/');
        let parentId = '';
        let targetDiv = null;
        let title = '';
        let where = 'afterbegin';

        Object.entries(domAr).forEach(([key, value]) => {
            if (key == 0) {
                where = (value == 'hc') ? 'afterbegin' : 'beforeend';
                parentId = value;
                let showtarget = (showRooms.includes(value) || value == 'infopanels') ? 'panel-autobtns':'archive';
                if (!roomNames[value]) { // unknown room, display it and add to config
                    roomNames[value] = value + ': no name available in config.js';
                    showtarget = 'panel-autobtns';
                }
                targetDiv = document.getElementById(showtarget);
            } else {
                targetDiv = document.getElementById(parentId + '-group-ul');
                parentId += '/' + value;
            }

            title = (roomNames[value]) ? roomNames[value]:' ' + value;

            if (document.getElementById(parentId + '-group') == null) {
                let togClass = 'toggle-content toggle-content-hidden';
                let togIconClass = '';

                // let groupClass = (showRooms.includes(value) || key > 0) ? '':'hard-hidden';

                if (locStore === true) {
                    if (JSON.parse(localStorage.getItem('foldout')).includes(parentId + '-group')) {
                        togClass = 'toggle-content';
                        togIconClass = 'fa-rotate-90';
                    }
                } else {
                    if (initialFold.includes(parentId + '-group')) {
                        togClass = 'toggle-content';
                        togIconClass = 'fa-rotate-90';
                    }
                }

                let button_txt = '';
                (buttons == true) ? button_txt = `<button type="button" data-pwr="${parentId}" data-buttontype="${buttonType}" class="device-onoff-btn">
                                <span class="device-state-text"></span>
                                <span class="fa"></span>
                                <span id="${parentId}-group-cnt" >&nbsp;</span>
                            </button>`:'';
                let headcss = 'group-head';
                (buttons == false) ? headcss = 'group-head-single':'';
                let html = `
                    <li id="${parentId}-group" name="${parentId}-group" class="device-group" data-state="off">
                        <div class="${headcss}">
                            <h2>
                                <a href="#" title="toggle" class="group-toggle" data-toggroupid="${parentId}-group-ul"><span class="fa fa-fw fa-caret-right ${togIconClass}"></span> ${title}</a>
                            </h2>
                            ${button_txt}
                        </div>
                        <ul id="${parentId}-group-ul" class="group-body ${togClass}">
                        </ul>
                    </li>`;

                targetDiv.insertAdjacentHTML(where, html);

                // if local Storage is available, sort rooms
                if (locStore === true && localStorage.getItem('showRooms')) {
                    let ar = JSON.parse(localStorage.getItem('showRooms'));
                    for (let item of ar) {
                        if (document.getElementById(item + '-group') !== null) {
                            document.getElementById('panel-autobtns').appendChild(document.getElementById(item + '-group'));
                        }
                    }
                    if (document.getElementById('infopanels-group') != null ) {
                        document.getElementById('panel-autobtns').appendChild(document.getElementById('infopanels-group'));
                    }
                }
            }
        });
    }

    // Lasershow for Cutteractivity
    function lasershow(message) {
        if (document.getElementById(message.name) == null ) {
            document.getElementById('content').insertAdjacentHTML('beforeend', `
                <div id="${message.name}" class="lasercutter" >
                    <div class="laser-beam beam1"></div>
                    <div class="laser-beam beam2"></div>
                    <div class="laser-beam beam3"></div>
                    <div class="laser-beam beam4"></div>
                </div>`);
        }
        if (document.getElementById(message.name + '_turnoff_box') == null && message.status == 1) {
            addInfoPanelGroup();
            let datastate = (message.status == '1') ? 'on':'off';
            document.getElementById('infopanels-group-ul').insertAdjacentHTML('beforeend', `
            <li id="${message.name}_turnoff_box" class="setting-button" data-state="${datastate}">
                <div class="setting-head">
                    <h3><span class="fa fa-fw fa-bolt setting-icon"></span> Laser is </h3>
                    <button type="button" data-buttontype="disable_laser" device-state="${datastate}" class="device-onoff-btn">
                        <span class="device-state-text"></span>
                        <span class="fa"></span>
                        <span></span>
                    </button>
                </div>
            </li>`);
        }

        if (document.getElementById(message.name + '_turnoff_box') != null && message.status == 0) {
            removeElemtBySelector(`#${message.name}_turnoff_box`);
        }

        document.getElementById(message.name).dataset.state = message.status;
        document.getElementById('logo').dataset.laserstate = message.status;
    }

    function refreshJingleList(deviceId) {
        sendMqttMessage(`${deviceId}/mpc`, 'jinglelist')
    }

    function addMpdControl(message) {
        let deviceInfos = message.destinationName.match(/^mpd\/(.+)\/([^/]+)\/info$/);
        let deviceId = 'mpd/' + deviceInfos[1] + '/' + deviceInfos[2];

        let mpdInfo = null;
        try {
            mpdInfo = JSON.parse(message.payloadString);
            // console.log(mpdInfo);
        } catch {
            console.log('JSON KAPUTT');
            return;
        }

        // ask the server for the current jinglelist on a separate topic
        refreshJingleList(deviceId)

        if (document.getElementById(deviceId) == null) {
            domGroupCreation(deviceInfos[1]);

            let targetId = document.getElementById(deviceInfos[1] + '-group-ul');
            if (targetId == null) {
                // console.log('No BTN created, not found targetId: ', targetId);
                return;
            }

            let selStr = '';
            for (var i = 0; i <= 100; i += 5) {
                selStr += `<option value="${i}">${i}</option>`;
            }

            targetId.insertAdjacentHTML('beforeend', `
                <li id="${deviceId}" class="device-mpd" >
                    <div class="">
                        <h3 class="mpd-headline"><marquee><span id="${deviceId}-title">${mpdInfo.Song}</span></marquee></h3>
                    </div>
                    <div class="device-info">
                        <div class="mpd-control-buttons">
                            <button type="button" id="${deviceId}-prev" class="" data-mpd="${deviceId}" data-buttontype="mpd-prev" title="play prev title">
                                <span class="fa fa-step-backward fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-seekback" class="" data-mpd="${deviceId}" data-buttontype="mpd-seekback" title="seek back 5%">
                                <span class="fa fa-backward fa-fw"></span>
                            </button>
                            <button type="button" class="mpd-play-button" data-mpd="${deviceId}" data-buttontype="mpd-play">
                                <span class="fa fa-music fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-seek" class="" data-mpd="${deviceId}" data-buttontype="mpd-seek" title="seek forward 5%">
                                <span class="fa fa-forward fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-next" class="" data-mpd="${deviceId}" data-buttontype="mpd-next" title="play next title">
                                <span class="fa fa-step-forward fa-fw"></span>
                            </button>
                            <div class="mpd-vol-button">
                                <span class="fa fa-volume-up fa-fw"></span>
                                <select id="${deviceId}-volume" data-mpd="${deviceId}">${selStr}</select>
                            </div>
                         </div>
                        <div class="mpd-state-buttons">
                            <button type="button" id="${deviceId}-random" class="" data-mpd="${deviceId}" data-buttontype="mpd-random" title="Play random">
                                <span class="fa fa-random fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-repeat" class="" data-mpd="${deviceId}" data-buttontype="mpd-repeat" title="repeat playlist">
                                <span class="fa fa-repeat fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-single" class="" data-mpd="${deviceId}" data-buttontype="mpd-single" title="play only single">
                                <span class="fa fa-star fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-consume" class="" data-mpd="${deviceId}" data-buttontype="mpd-consume" title="remove title from playlist after played">
                                <span class="fa fa-fire fa-fw"></span>
                            </button>
                            <button type="button" id="${deviceId}-jingle" class="" data-mpd="${deviceId}" data-buttontype="mpd-jingle" title="random jingle">
                                <span class="icon-superscript">
                                    <i class="fa fa-bullhorn"></i>
                                    <i class="fa fa-random"></i>
                                </span>
                            </button>
                            <button type="button" data-togtarget="${deviceId}-info"><span class="fa fa-info-circle fa-fw"></span>
                            </button>

                        </div>
                        <div for="${deviceId}-ytdl-url" class="mpd-yt-buttons">
                            <input type="url" id="${deviceId}-ytdl-url" placeholder="youtubedl url">
                            <button type="button" class="mpd-ytdl-button" data-mpd="${deviceId}" data-buttontype="mpd-ytdl" title="insert any Media Link (YouTube Soundcloud Vimeo ...) - will be Next in line">
                                <span class="fa fa-youtube-play fa-fw"></span>
                                <span class="fa fa-soundcloud fa-fw"></span>
                            </button>
                        </div>
                        <div>
                            <input type="range" min="0" max="100" value="${mpdInfo.volume.replace(/%()/g,'')}" data-mpd="${deviceId}" id="${deviceId}-slider">
                        </div>
                        <div id="${deviceId}-info" class="device-info toggle-content toggle-content-hidden">
                            <p class="mpd-location">File: <span id="${deviceId}-location"></span></p>
                        </div>

                    </div>
                </li>
                `);

            // listener for volume change
            document.getElementById(deviceId + '-volume').addEventListener('change', function(e){
                sendMqttMessage(document.getElementById(deviceId + '-volume').dataset.mpd + '/mpc', 'volume ' + e.target.value + '');
            });
            document.getElementById(deviceId + '-slider').addEventListener('change', function(e){
                sendMqttMessage(document.getElementById(deviceId + '-slider').dataset.mpd + '/mpc', 'volume ' + e.target.value + '');
            });
        }

        document.getElementById(deviceId + '-random').dataset.state = mpdInfo.random;
        document.getElementById(deviceId + '-repeat').dataset.state = mpdInfo.repeat;
        document.getElementById(deviceId + '-single').dataset.state = mpdInfo.single;
        document.getElementById(deviceId + '-consume').dataset.state = mpdInfo.consume;

        let playBtn = document.querySelector(`[data-buttontype="mpd-play"][data-mpd="${deviceId}"]`);
        if (mpdInfo.state == '[stopped]') {
            document.getElementById(deviceId).dataset.mpdState = 'stop';
            playBtn.innerHTML = '<span class="fa fa-stop fa-fw"></span>';
        } else if (mpdInfo.state == '[playing]') {
            document.getElementById(deviceId).dataset.mpdState = 'play';
            playBtn.innerHTML = '<span class="fa fa-play fa-fw"></span>';
        } else if (mpdInfo.state == '[paused]') {
            document.getElementById(deviceId).dataset.mpdState = 'pause';
            playBtn.innerHTML = '<span class="fa fa-pause fa-fw"></span>';
        }

        if (mpdInfo.Song != document.getElementById(deviceId + '-title').innerHTML) {
            document.getElementById(deviceId + '-title').innerHTML = mpdInfo.Song;
        }

        if (mpdInfo.location != document.getElementById(deviceId + '-location').innerHTML) {
            document.getElementById(deviceId + '-location').innerHTML = mpdInfo.location.substring(0, 160);
        }

        let vol = Math.floor(parseInt(mpdInfo.volume.replace('%', '')) / 5 ) * 5;
        (isNaN(vol)) ? vol = 0:'';
        document.getElementById(`${deviceId}-volume`).querySelector(`[value="${vol}"]`).selected = true;
        document.getElementById(`${deviceId}-slider`).value = vol;
    }

    function parseJingleName(filename) {
        let splits = filename.split('/')
        return splits[splits.length-1]
            .replace(/\.(mp3|ogg|flac|m4a)$/, '')
            .split(/[^a-zA-Z0-9'-]+/)
            .filter(word => word.length)
            .map(word => word[0].toUpperCase() + word.substring(1).toLowerCase())
            .join(' ');
    }

    function addJingleBoard(message) {
        let [, room, mpdNumber] = message.destinationName.match(/^mpd\/(.+)\/([^/]+)\/jinglelist$/);
        let deviceId = `mpd/${room}/${mpdNumber}`;
        let jingles = message.payloadString.split('\n').map(filename => ({
            filename,
            name: parseJingleName(filename),
        }));
        jingles = localeSort(jingles, j => j.name)

        let deviceWrapper = document.getElementById(deviceId)
        let buttonsWrapper = deviceWrapper && deviceWrapper.querySelector('.mpd-state-buttons')

        // If the deviceWrapper or buttonsWrapper does not exist, no use in
        // doing anything.
        if (!buttonsWrapper) {
            return
        }

        // If the button does not exist already, add it.
        if (!buttonsWrapper.querySelector('button[data-buttontype="mpd-jingleboard"]')) {
            buttonsWrapper.insertAdjacentHTML('beforeend', `
                <button type="button" id="${deviceId}-jingleboard-button" data-mpd="${deviceId}" data-buttontype="mpd-jingleboard" title="Open jingle board">
                    <span class="fa fa-bullhorn fa-fw"></span>
                </button>
            `);
        }

        let jingleboardWrapper = document.getElementById(`${deviceId}-jingleboard`)

        if (!jingleboardWrapper) {
            document.body.insertAdjacentHTML('beforeend', `
                <div id="${deviceId}-jingleboard" data-mpd="${deviceId}" class="jingleboard" data-visible="no">
                    <div class="jinglelist"></div>
                </div>
            `);

            jingleboardWrapper = document.getElementById(`${deviceId}-jingleboard`)
        }

        let jingleButtons = jingles.map(jingle => (
            `
                <button type="button" class="jingleboard-button" data-buttontype="mpd-jingle" data-mpd="${deviceId}" data-jingle="${jingle.filename}">
                    <span class="text">
                        ${jingle.name}
                    </span>
                </button>
            `
        )).join('');

        jingleboardWrapper.querySelector('.jinglelist').innerHTML = jingleButtons
    }

    function openJingleBoard(mpd) {
        document.querySelector(`.jingleboard[data-mpd="${mpd}"]`).dataset.visible = 'yes'
    }

    function closeJingleBoard() {
        document.querySelector(`.jingleboard[data-visible="yes"]`).dataset.visible = 'no'
    }

    /*
    * Generalized RGB Controlbuttons, See for example: https://git.cccfr.de/smtw/mqtt-rgb-esp32-treppe
    */
    function rgbCtrl(message) {
        let deviceInfos = message.destinationName.match(/^rgb\/(.+)\/([^\/]+)\/stat\/([^\/]+)$/);
        let deviceId = 'rgb/' + deviceInfos[1] + '/' + deviceInfos[2];

        if (document.getElementById(deviceId) == null && deviceInfos[3] == 'LWT' && message.payloadString.toLowerCase() == 'online') {

            domGroupCreation(deviceInfos[1]);

            let targetId = document.getElementById(deviceInfos[1] + '-group-ul');
            if (targetId == null) {
                return;  // console.log('No BTN created, not found targetId: ', targetId);
            }

            targetId.insertAdjacentHTML('beforeend', `
            <li id="${deviceId}" class="device-rgb-led" data-devicetopic="${deviceId}" data-networkstate="online" data-state="off">
                <div class="group-head">
                    <h3><a href="#" title="toggle" data-togtarget="${deviceId}-info"><span class="fa fa-lightbulb-o fa-fw device-icon"></span> ${deviceInfos[2]}</a></h3>
                    <button type="button" data-pwr="${deviceId}" data-buttontype="rgb_power" class="device-onoff-btn">
                        <span class="device-state-text"></span>
                        <span class="fa"></span>
                        <span></span>
                    </button>
                </div>
                <div id="${deviceId}-info" class="group-body device-info toggle-content toggle-content-hidden">
                </div>
            </li>`);

            sendMqttMessage('' + deviceId + '/cmd/info' , ''); // request actual state
            return;
        }

        if (deviceInfos[3] == 'LWT' && message.payloadString.toLowerCase() == 'offline') {
            document.getElementById(deviceId).setAttribute('data-networkstate', 'offline');
            return;
        }

        if (deviceInfos[3].toLowerCase() == 'power') {
            if (message.payloadString.toLowerCase() == 'on') {
                document.getElementById(deviceId).setAttribute('data-state', 'on');
                switchGroupButtonState(document.getElementById(deviceInfos[1] + '-group'), 'on');
            } else {
                document.getElementById(deviceId).setAttribute('data-state', 'off');
                switchGroupButtonState(document.getElementById(deviceInfos[1] + '-group'), 'off');
            }
        }

        if (deviceInfos[3] == 'info') {
            let json;
            try {
                json = JSON.parse(message.payloadString);
            } catch(e) {
                // JSON parsing failed
                return;
            }

            for (let item of json.btns) {
                rgbAutoButtons(deviceId, item);
            }
            if (document.getElementById(deviceId + '-info-ip') == null) {
                document.getElementById('' + deviceId + '-info').insertAdjacentHTML('beforeend', `<p id="${deviceId}-info-ip" class="device-ip">IP: ${json.ip}</p>`);
            }
        }
    }

    function rgbAutoButtons(deviceId, item) {
        if (item.tp == 'rgb') {
            let value = rgbToHex(item.v[0], item.v[1], item.v[2]);
            if (document.getElementById(deviceId + '-' + item.cmd) == null) {
                document.getElementById(deviceId + '-info').insertAdjacentHTML('beforeend', `<label for="${deviceId}-${item.cmd}">${item.ti} </label><input data-topic="${deviceId}/cmd/${item.cmd}" data-buttontype="rgb_rgb" type="color" name="rgb color" value="${value}" id="${deviceId}-${item.cmd}" >`);
                let triggerTarget = document.getElementById(`${deviceId}-${item.cmd}`);
                triggerTarget.addEventListener('change', function(){
                    sendMqttMessage(triggerTarget.dataset.topic, `[${hexToRgb(triggerTarget.value)}]`);
                }, false);
            } else {
                document.getElementById(deviceId + '-' + item.cmd).value = value;
            }
        }

        if (item.tp == 'slide') {
            if (document.getElementById(deviceId + '-' + item.cmd) == null) {
                document.getElementById(deviceId + '-info').insertAdjacentHTML('beforeend', `<label for="${deviceId}-${item.cmd}">${item.ti} </label><input data-topic="${deviceId}/cmd/${item.cmd}" data-buttontype="rgb_brightness" type="range" name="rgb brightness" min="0" max="100" value="${item.v}" id="${deviceId}-${item.cmd}" >`);
                let triggerTarget = document.getElementById(`${deviceId}-${item.cmd}`);
                triggerTarget.addEventListener('change', function(){
                    sendMqttMessage(triggerTarget.dataset.topic, triggerTarget.value);
                }, false);
            } else {
                document.getElementById(deviceId + '-' + item.cmd).value = item.v;
            }
        }

        if (item.tp == 'pd') {
            if (document.getElementById(deviceId + '-' + item.cmd) == null) {
                let options = `<label for="${deviceId}-${item.cmd}">${item.ti} </label><select name="rgb Pulldown" data-topic="${deviceId}/cmd/${item.cmd}" data-buttontype="rgb_presets" id="${deviceId}-${item.cmd}">`;
                for (let kv_i in item.kv) {
                    options += `<option value="${kv_i}">${item.kv[kv_i]}</option>`;
                }
                options += '</select>';
                document.getElementById(deviceId + '-info').insertAdjacentHTML('beforeend', options);
                document.getElementById(deviceId + '-' + item.cmd).value = item.v;
                let triggerTarget = document.getElementById(`${deviceId}-${item.cmd}`);
                triggerTarget.addEventListener('change', function(){
                    sendMqttMessage(triggerTarget.dataset.topic, triggerTarget.value);
                }, false);
            } else {
                document.getElementById(deviceId + '-' + item.cmd).value = item.v;
            }
        }
    }

    function rgbPowerToggle(target, state) {
        sendMqttMessage(target + '/cmd/power' , ( ( state == 'off' ) ? 'on':'off' ) );
    }

    var hexToRgb = function(hex) {
        hex = hex.replace('#', '');
        var bigint = parseInt(hex, 16);
        var r = (bigint >> 16) & 255;
        var g = (bigint >> 8) & 255;
        var b = bigint & 255;

        return [r, g, b].join();
    }

    const rgbToHex = (r, g, b) => '#' + [r, g, b].map(x => x.toString(16).padStart(2, '0')).join('');

    /*
    * THED GAME CONTROL
    */
    function thedGame(message) {
        let deviceId = 'experimental/thed/grossundunpraktisch';

        if (message.payloadString == 'offline') {
            return;
        }

        domGroupCreation('hc');

        if (document.getElementById(deviceId) == null) {
            document.getElementById('hc-group-ul').insertAdjacentHTML('beforeend', `
            <li id="${deviceId}" class="device-thed" data-devicetopic="${deviceId}" data-networkstate="online">
                <h3><span class="fa fa-space-shuttle fa-fw"></span> Thed</h3>
                <button type="button" data-ctrl="left" data-buttontype="thed-control" accesskey="q">
                    <span class="fa fa-play fa-rotate-180 fw"></span>
                </button>
                <button type="button" data-ctrl="attack" data-buttontype="thed-control" accesskey="w">
                    <span class="fa fa-crosshairs fw"></span>
                </button>
                <button type="button" data-ctrl="right" data-buttontype="thed-control" accesskey="e">
                    <span class="fa fa-play fw"></span>
                </button>
            </li>`);
        }
    }

    /*
    * Screen control
    */
    function lwndScreenCtrl(message) {
        let deviceInfos = message.destinationName.match(/^lwnd\/(.+)\/([^/]+)\/([^/]+)$/);
        let deviceId = 'lwnd/' + deviceInfos[1] + '/' + deviceInfos[2];

        if (document.getElementById(deviceId) == null && message.payloadString.toLowerCase() != 'offline') {

            domGroupCreation('hc');

            if (document.getElementById('hc-group-ul') != null) {
                document.getElementById('hc-group-ul').insertAdjacentHTML('beforeend', `
                <li id="${deviceId}" class="device-lwnd" data-devicetopic="${deviceId}" data-networkstate="online">
                    <div class="lwnd-head">
                    <h3><span class="fa fa-desktop fa-rotate-180 fa-fw"></span> ${deviceInfos[2]}</h3>
                    <button type="button" data-lwndctrl="down" data-buttontype="lwnd-control">
                        <span class="fa fa-play fa-rotate-90 fw"></span>
                    </button>
                    <button type="button" data-lwndctrl="stop" data-buttontype="lwnd-control">
                        <span class="fa fa-stop fw"></span>
                    </button>
                    <button type="button" data-lwndctrl="up" data-buttontype="lwnd-control">
                        <span class="fa fa-play fa-rotate-270 fw"></span>
                    </button>
                    </div>
                </li>`);
            }
            return;
        }

        if (message.payloadString.toLowerCase() == 'offline' && document.getElementById(deviceId) != null) {
            document.getElementById(deviceId).setAttribute('data-networkstate', 'offline');
        }
    }

    function switchGroupButtonState(element, state) {
        if (element == null || element.id == 'panel-autobtns') {
            return;
        }

        // check if sate of other devices in a group = on, dont change group state then
        if (state == 'off'
            && Array.from(element.querySelectorAll(':scope li:not(.device-group)[data-state="on"]')).length > 0) {
            switchGroupButtonState(element.parentElement, state);
            return;
        }

        if (element.classList.contains('device-group')) {
            element.setAttribute('data-state', state);
        }

        switchGroupButtonState(element.parentElement, state);
    }

    /*
    * OBI SOCKET FOO
    */
    // settings and stats for obi_sockets
    function obiSocketStat(message) {
        let deviceInfos = message.destinationName.match(/^pwr\/(.+)\/([^/]+)\/stat\/([^/]+)$/);
        let targetDom = document.getElementById('pwr/' + deviceInfos[1] + '/' + deviceInfos[2]);

        if(targetDom == null) {
            // console.log('statObiSocket: missing: ', targetDom);
            return;
        }

        if (deviceInfos[3].toLowerCase() == 'power') {
            if (message.payloadString.toLowerCase() == 'on') {
                targetDom.setAttribute('data-state', 'on');
                switchGroupButtonState(document.getElementById(deviceInfos[1] + '-group'), 'on');
            } else {
                targetDom.setAttribute('data-state', 'off');
                switchGroupButtonState(document.getElementById(deviceInfos[1] + '-group'), 'off');
            }
        }

        if (deviceInfos[3].toLowerCase() == 'status5') {
            targetDom = document.getElementById('pwr/' + deviceInfos[1] + '/' + deviceInfos[2] + '-link');
            let netStat = JSON.parse(message.payloadString);
            if (netStat.StatusNET.IPAddress !== undefined) {
                targetDom.href = 'http://' + netStat.StatusNET.IPAddress;
            }
        }
    }

    // process tele commands for obi_socket devices
    function obiSocketTele(message) {
        let deviceInfos = message.destinationName.match(/^pwr\/(.+)\/([^/]+)\/tele\/LWT$/);
        let deviceId = 'pwr/' + deviceInfos[1] + '/' + deviceInfos[2];

        if (document.getElementById(deviceId) == null) {
            if (message.payloadString.toLowerCase() !== 'online') {
                return;
            }
            domGroupCreation(deviceInfos[1]);

            let targetId = document.getElementById(deviceInfos[1] + '-group-ul');
            if (targetId == null) {
                // console.log('No BTN created, not found targetId: ', targetId);
                return;
            }

            let icon = 'fa-plug';

            // (deviceInfos[2] == 'Barista-Antifascista') ? icon = 'fa-coffee':'';

            targetId.insertAdjacentHTML('beforeend', `
                <li id="${deviceId}" class="device-obi-socket" data-state="off" data-networkstate="online">
                    <div class="device-obi-head">
                        <h3>
                            <span class="fa ${icon} fa-fw device-obi-icon"></span> ${deviceInfos[2]}
                            <a href="#" title="Toggle Device Infos" data-togtarget="${deviceId}-info" class="device-obi-infobtn">
                                <span class="fa fa-info-circle fa-fw"></span>
                            </a>
                        </h3>
                    <button type="button" data-pwr="${deviceId}" data-buttontype="obi_socket" class="device-onoff-btn">
                        <span class="device-state-text"></span>
                        <span class="fa"></span>
                        <span></span>
                    </button>
                    </div>
                    <div id="${deviceId}-info" class="device-info toggle-content toggle-content-hidden">
                        <p><a id="${deviceId}-link" target="_blank" href="#" title="">zur config</a> <span class="small">Topic: ${deviceId} | Typ: <a href="https://github.com/arendst/Tasmota" target="_blank" title="Tasmota auf Github">Tasmota Steckdose</a></span></p>
                        <ul id="${deviceId}-sensor"></ul>
                    </div>
                </li>
                `);

            sendMqttMessage('' + deviceId + '/cmnd/power' , ''); // request actual state
            sendMqttMessage('' + deviceId + '/cmnd/STATUS' , '5'); // get some network infos (ip etc)
        }

        if (message.payloadString.toLowerCase() == 'online') {
            let targetId = document.getElementById(deviceId);
            targetId.disabled = false;
            targetId.setAttribute('data-networkstate', 'online');
        }

        if (message.payloadString.toLowerCase() == 'offline') {
            let targetId = document.getElementById(deviceId);
            targetId.disabled = true;
            targetId.setAttribute('data-networkstate', 'offline');
            targetId.setAttribute('data-state', 'off');
            switchGroupButtonState(document.getElementById(deviceInfos[1] + '-group'), 'off');
        }
    }

    function pwrSensor(message) {
        let deviceInfos = message.destinationName.match(/^pwr\/(.+)\/([^/]+)\/tele\/SENSOR$/);
        let deviceId = 'pwr/' + deviceInfos[1] + '/' + deviceInfos[2];
        let ar = JSON.parse(message.payloadString);
        if (!ar.ENERGY) {
            return;
        }
        target = document.getElementById(`${deviceId}-sensor`)
        if (target == null) { // 
            return;
        }
        target.innerHTML = `
        <li>TotalStartTime: ${ar.ENERGY.TotalStartTime}</li>
        <li>Total: ${ar.ENERGY.Total} kWh</li>
        <li>Yesterday: ${ar.ENERGY.Yesterday} kWh</li>
        <li>Today: ${ar.ENERGY.Today} kWh</li>
        <li>Period: ${ar.ENERGY.Period}</li>
        <li>Power: ${ar.ENERGY.Power} W</li>
        <li>ApparentPower: ${ar.ENERGY.ApparentPower} VA</li>
        <li>ReactivePower: ${ar.ENERGY.ReactivePower} VAr</li>
        <li>Factor: ${ar.ENERGY.Factor}</li>
        <li>Voltage: ${ar.ENERGY.Voltage} V</li>
        <li>Current: ${ar.ENERGY.Current} A</li>`;
    }

    function obiSocketToggle(target, state) {
        sendMqttMessage(target + '/cmnd/power' , ( ( state == 'off' ) ? 'ON':'OFF' ) );
    }

    document.addEventListener('touchstart', onclick);
    /*
    * 1 CLICK LISTENER TO RULE THEM ALL!!11!!
    */
    document.addEventListener('click',function onclick(e){
        if (e.target.id == "client-conn-status-button") {
            location.reload();
        }

        // device infos or for multi
        if (e.target.hasAttribute('data-togtarget')) {
            e.preventDefault();
            document.getElementById(e.target.dataset.togtarget).classList.toggle('toggle-content-hidden');
            if (e.target.firstChild.hasOwnProperty('classList') && e.target.firstChild.classList.contains('fa-caret-right')) {
                e.target.firstChild.classList.toggle('fa-rotate-90');
            }
        }

        // fold out / in groups
        if (e.target.hasAttribute('data-toggroupid')) {
            e.preventDefault(); //prevent from jumping inpage
            let ele = document.getElementById(e.target.dataset.toggroupid);
            ele.classList.toggle('toggle-content-hidden');
            if (ele.classList.contains('toggle-content-hidden')) {
                // remove from list
                locStoreJsonRemove('foldout', ele.id.replace('-ul', ''));
            } else {
                // add to storage
                locStoreJsonAdd('foldout', ele.id.replace('-ul', ''));
            }
            if (e.target.firstChild.classList.contains('fa-caret-right')) {
                e.target.firstChild.classList.toggle('fa-rotate-90');
            }
        }

        // hide jingleboard
        if (e.target.classList.contains('jingleboard')) {
            e.preventDefault();
            e.target.dataset.visible = 'no';
        }

        // click on button with data-buttontype attribute
        if (e.target.hasAttribute('data-buttontype')) {

            // clicked a single OBI power button
            if (e.target.dataset.buttontype == 'obi_socket') {
                obiSocketToggle(e.target.dataset.pwr, document.getElementById(e.target.dataset.pwr).dataset.state, true);
                return;
            }

            // clicked a single RGB power button
            if (e.target.dataset.buttontype == 'rgb_power') {
                rgbPowerToggle(e.target.dataset.pwr, document.getElementById(e.target.dataset.pwr).dataset.state, true);
                return;
            }

            // clicked a group OBI power button
            if (e.target.dataset.buttontype == 'obi_socket_group') {
                let domAr = document.getElementById(e.target.dataset.pwr + '-group-ul').getElementsByTagName('button');
                let state = document.getElementById(e.target.dataset.pwr + '-group').dataset.state;
                Object.entries(domAr).forEach(([key, value]) => {
                    if (value.dataset.buttontype == 'obi_socket') {
                        obiSocketToggle(value.dataset.pwr, state);
                    }
                    if (value.dataset.buttontype == 'rgb_power') {
                        rgbPowerToggle(value.dataset.pwr, state);
                    }
                });
                return;
            }

            // clicked screen motor control element
            if (e.target.dataset.buttontype == 'lwnd-control') {
                sendMqttMessage(e.target.parentElement.parentElement.dataset.devicetopic + '/cmd', e.target.dataset.lwndctrl);
                return;
            }

            // clicked thed game control elements
            if (e.target.dataset.buttontype == 'thed-control') {
                sendMqttMessage(e.target.parentElement.dataset.devicetopic + '/cmd', e.target.dataset.ctrl);
                return;
            }

            // Doorstate
            if (e.target.dataset.buttontype == 'door-button') {
                sendMqttMessage('door/cmd' , ( ( document.getElementById('doorstate').dataset.state == 'close' ) ? '1':'0' ) );
                return;
            }

            // MPD Control - Play
            if (e.target.dataset.buttontype == 'mpd-play') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'toggle' );
                return;
            }

            // MPD Control - Toggle Repeat
            if (e.target.dataset.buttontype == 'mpd-repeat') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'repeat' );
                return;
            }

            // MPD Control - Toggle Random
            if (e.target.dataset.buttontype == 'mpd-random') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'random' );
                return;
            }

            // MPD Control - Toggle single
            if (e.target.dataset.buttontype == 'mpd-single') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'single' );
                return;
            }

            // MPD Control - Toggle consume
            if (e.target.dataset.buttontype == 'mpd-consume') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'consume' );
                return;
            }

            // MPD Control - Next
            if (e.target.dataset.buttontype == 'mpd-next') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'next' );
                return;
            }

            // MPD Control - prev
            if (e.target.dataset.buttontype == 'mpd-prev') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'prev' );
                return;
            }


            // MPD Control - seekback
            if (e.target.dataset.buttontype == 'mpd-seekback') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'seek -5%' );
                return;
            }

            // MPD Control - seek
            if (e.target.dataset.buttontype == 'mpd-seek') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'seek  +5%' );
                return;
            }

            // MPD Control - jingle
            if (e.target.dataset.buttontype == 'mpd-jingle') {
                let cmd = 'jingle'
                if (e.target.dataset.jingle) {
                    cmd += ' ' + e.target.dataset.jingle
                }
                sendMqttMessage(e.target.dataset.mpd + '/mpc', cmd);
                closeJingleBoard();
                e.preventDefault()
                e.stopPropagation()
                return;
            }

            if (e.target.dataset.buttontype == 'mpd-jingleboard') {
                openJingleBoard(e.target.dataset.mpd)
                return;
            }

            // MPD Control - youtube dl
            if (e.target.dataset.buttontype == 'mpd-ytdl' && document.getElementById(e.target.dataset.mpd + '-ytdl-url').value != '') {
                sendMqttMessage(e.target.dataset.mpd + '/mpc' , 'ytdl ' + document.getElementById(e.target.dataset.mpd + '-ytdl-url').value );
                document.getElementById(e.target.dataset.mpd + '-ytdl-url').value = '';
                return;
            }

            // room configuration
            if (e.target.dataset.buttontype == 'showroom') {
                let state = (document.getElementById('room-show-' + e.target.dataset.showroom).dataset.state == 'off') ? 'on':'off';
                document.getElementById('room-show-' + e.target.dataset.showroom).dataset.state = state;
                if (document.getElementById(e.target.dataset.showroom + '-group') !== null) {
                    if (state == 'on') {
                        document.getElementById('panel-autobtns').appendChild(document.getElementById(e.target.dataset.showroom + '-group'));
                        locStoreJsonAdd('showRooms', e.target.dataset.showroom);
                        if (document.getElementById('infopanels-group') != null ) {
                            document.getElementById('panel-autobtns').appendChild(document.getElementById('infopanels-group'));
                        }
                    } else {
                        document.getElementById('archive').appendChild(document.getElementById(e.target.dataset.showroom + '-group'));
                        locStoreJsonRemove('showRooms', e.target.dataset.showroom);
                    }
                }
                return;
            }

            // topic subscription
            if (e.target.dataset.buttontype == 'showtopic') {
                let state = (document.getElementById('topic-show-' + e.target.dataset.showtopicclean).dataset.state == 'off') ? 'on':'off';
                document.getElementById('topic-show-' + e.target.dataset.showtopicclean).dataset.state = state;
                if (state == 'on') {
                    locStoreJsonAdd('subscibedTopics', e.target.dataset.showtopic);
                    topicSubscribe(e.target.dataset.showtopic);
                } else {
                    locStoreJsonRemove('subscibedTopics', e.target.dataset.showtopic);
                    topicUnsubscribe(e.target.dataset.showtopic);
                }
                return;
            }

            // Laser abschalten
            if (e.target.dataset.buttontype == 'disable_laser') {
                if (!confirm("Are you shure to turn off the laser?")) {
                    return;
                }
                sendMqttMessage('info/laser/hot', '{"name":"laserpi","status":0,"uptime":"0, 0 users,  load average: 0"}', true);
            }

            console.log('not found: ', e.target.dataset.buttontype);
        }

        // open / close Infoscreen panel
        if (e.target.id == "infoscreen-btn") {
            e.preventDefault();
            document.getElementById('panel-infoscreen').classList.toggle('hard-hidden');
            showInfoboard = !document.getElementById('panel-infoscreen').classList.contains('hard-hidden')
        }
    });

    function addRoomSettings() {
        addInfoPanelGroup();
        domGroupCreation('infopanels/Räume', false);
        let cont = ``;
        Object.entries(roomNames).forEach(([key, value]) => {
            if (value == 'infopanels') {
                return;
            }
            let checked = (showRooms.includes(key)) ? 'on':'off';
            cont += `<li id="room-show-${key}" class="setting-button" data-state="${checked}">
            <div class="setting-head">
                <h3><span class="fa fa-fw fa-social-home setting-icon"></span> ${value}</h3>
                <button type="button" data-showroom="${key}" data-buttontype="showroom" class="device-onoff-btn">
                    <span class="device-state-text"></span>
                    <span class="fa"></span>
                    <span></span>
                </button>
           </div>
            </li>`;
        })
        document.getElementById('infopanels/Räume-group-ul').insertAdjacentHTML('beforeend', cont);
    }

    function addTopicSettings() {
        addInfoPanelGroup();
        domGroupCreation('infopanels/Topics', false);
        let cont = ``;
        Object.entries(monTopics).forEach(([key, value]) => {
             let checked = (subscibedTopics.includes(value)) ? 'on':'off';
             let cleanValue = value.replace('/#', '').replace('/+', '').trim();
             cont += `<li id="topic-show-${cleanValue}" class="setting-button" data-state="${checked}">
             <div class="setting-head">
                 <h3><span class="fa fa-fw fa-social-home setting-icon"></span> ${cleanValue}</h3>
                 <button type="button" data-showtopic="${value}" data-showtopicclean="${cleanValue}" data-buttontype="showtopic" class="device-onoff-btn">
                     <span class="device-state-text"></span>
                     <span class="fa"></span>
                     <span></span>
                 </button>
            </div>
             </li>`;
        })
        cont += `</form></li>`;
        document.getElementById('infopanels/Topics-group-ul').insertAdjacentHTML('beforeend', cont);
    }
    connect(); // initial start
}

function locStoreJsonAdd(item, value) {
    if (locStore === true) {
        let ar = JSON.parse(localStorage.getItem(item));
        if (!ar.includes(value)) {
            ar.push(value);
        }
        localStorage.setItem(item, JSON.stringify(ar));
    }
}

function locStoreJsonRemove(item, value) {
    if (locStore === true) {
        let ar = JSON.parse(localStorage.getItem(item));
        var index = ar.indexOf(value);
        if (index !== -1) ar.splice(index, 1);
        localStorage.setItem(item, JSON.stringify(ar));
    }
}

// check wehter localstorage is available or not
function storageAvailable() {
    var storage;
    try {
        storage = window['localStorage'];
        var x = '__storage_test__';
        storage.setItem(x, x);
        storage.removeItem(x);
        return true;
    }
    catch(e) {
        return e instanceof DOMException && (
            // everything except Firefox
            e.code === 22 ||
            // Firefox
            e.code === 1014 ||
            // test name field too, because code might not be present
            // everything except Firefox
            e.name === 'QuotaExceededError' ||
            // Firefox
            e.name === 'NS_ERROR_DOM_QUOTA_REACHED') &&
            // acknowledge QuotaExceededError only if there's something already stored
            (storage && storage.length !== 0);
    }
}
